# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
class Hermes::Api::Dmo::ApplicantsController < Hermes::Api::Dmo::ApplicationController
  require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

  class << self
    def mutate_params
      {
        actual_housing_details: { type: 'Hash'},
        main_employer_name: { type: 'String'},
        owned_housing_details: { type: 'Hash'},
        postal_details: { type: 'Hash'},
        foreign_workplace_postal_code: { type: 'String'},
        workplace_insee_code: { type: 'String'},
        workplace_postal_code: { type: 'String'},
        workplace_locality: { type: 'String'},
        foreign_workplace_locality: { type: 'String'},
      }
    end 
    
    def acceptable_params
      {
        amount_year_minus_one: { type: 'Float'},
        amount_year_minus_two: { type: 'Float'},
        apl_amount: { type: 'Float'},
        asked_children_nb: { type: 'Integer'},
        asked_locataire_hlm: { type: 'Boolean'},
        asked_priority_public: { type: 'Boolean'},
        birth_date: { type: 'Date'},
        cellphone: { type: 'String'},
        civility: { type: 'String'},
        command_to_leave_date: { type: 'Date'},
        completed_requests_nb: { type: 'Integer'},
        disability: { type: 'String'},
        email_advances_assistant: { type: 'String'},
        email: { type: 'String'},
        event_notification: { type: 'Boolean'},
        family_situation_validated: { type: 'Boolean'},
        family_situation: { type: 'String'},
        first_name: { type: 'String'},
        foreign_workplace_country: { type: 'String'},
        foreign_workplace_locality: { type: 'String'},
        has_other_employers: { type: 'Boolean'},
        homephone: { type: 'String'},
        housing_request_ids: { type: 'Array'},
        is_owner: { type: 'Boolean'},
        last_name: { type: 'String'},
        main_employer_siret: { type: 'String'},
        main_user_id: { type: 'String'},
        mobility: { type: 'Boolean'},
        monthly_charges: { type: 'Hash'},
        monthly_incomes: { type: 'Hash'},
        mutation: { type: 'Boolean'},
        nationality: { type: 'String'},
        nud: { type: 'String'},
        on_going_request_nb: { type: 'Integer'},
        patronymic_name: { type: 'String'},
        priority_public: { type: 'Boolean'},
        pro_or_home_tel_number: { type: 'String'},
        profession_details: { type: 'Hash'},
        renewal_date: { type: 'DateTime'},
        ressources_details: { type: 'Hash'},
        single_parent: { type: 'Boolean'},
        situation_tax_treshold: { type: 'Integer'},
        sms_notification: { type: 'Boolean'},
        sne_profile_creation_date: { type: 'DateTime'},
        ss_id: { type: 'String'},
        status_updated_at: { type: 'DateTime'},
        status: { type: 'String'},
        tax_revenues: { type: 'Float'},
        temporary_id: { type: 'String'},
        unemployment_period_date_end: { type: 'Date'},
        unemployment_period: { type: 'Boolean'},
        valid_children_nb: { type: 'Integer'},
        valid_locataire_hlm: { type: 'Boolean'},
        valid_priority_public: { type: 'Boolean'},
        work_contract_type: { type: 'String'},
        workphone: { type: 'String'},
        workplace_locality: { type: 'String'},
        year_minus_one: { type: 'String'},
        year_minus_two: { type: 'String'},
      }.merge(self.mutate_params).merge(super)
    end 

    def valid_operators
      %w[$in $nin $ne $gt $gte $lt $lte $eq $regex $all]
    end
  end

  def index
    acceptable_params *self.class.acceptable_params.keys
    valid_operators *self.class.valid_operators
    super
  end

  def user_is_allowed_to_remove_entity
    @instance.writable_by?(@user) || @instance.entity&.writable_by?(@user)
  end

  def all_eligibile_offers
    eligibility = JSON.parse(@instance.eligibility_query).each_with_object({}) do |(key, val), h|
      h[key.to_sym] = val
    end
    filter = build_query(Hermes::HousingOffer).deep_merge(eligibility)

    Hermes::HousingOffer.where(eligibility).where(build_query(Hermes::HousingOffer)).order(order_params)
  end

  def update
    return render_json_error(:not_found, :object_not_found) unless @instance
    return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && (@instance.writable_by?(@user) || @instance&.entity.writable_by?(@user))
    # return render_error_not_mutable(not_mutable) unless not_mutable.empty?

    [
      :actual_housing_details,
      :postal_details,
      :owned_housing_details,
    ].each do |key|
      if params[:data][:attributes][key]
        only_accepted_keys = params[:data][:attributes][key].slice(:address_insee_code, :postal_code, :foreign_district, :district)
        params[:data][:attributes][key] = (@instance[key] || {}).merge(only_accepted_keys) 
      end

    end 

    param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
    mutate_valid_params[:user_id] = param_user if param_user && @user.has_role?('Administrator')
    return render_json_error(:parameters_not_present, :no_valid_parameters) unless mutate_valid_params.present?
    Rails.logger.debug "TEST valid_params are #{mutate_valid_params}"
    return respond_with_errors(@instance) unless @instance.update(mutate_valid_params) && @instance.save

    render(jsonapi: @instance, serializer: @serializer)
  end

  def eligible_offers
    return render_json_error(:not_found, :instance_not_found) unless @instance

    if params[:reserving_siren].present?
      rh = Hermes::ReservationHolder.find_by(siren: params[:reserving_siren])
      params[:reserving_siren] = { '$in' => rh.siren } if rh 
    end 

    acceptable_params *Hermes::Api::Dmo::HousingOffersController.acceptable_params
    regex_params *Hermes::Api::Dmo::HousingOffersController.regex_params
    valid_operators *VALID_OPERATORS
    options = {}

    paginate(
      all_eligibile_offers,
      each_serializer: Hermes::HousingOfferSerializer,
      **options
    )
  end

  def find_by_nud
    @instance = @model.find_or_create_by_nud(current_user, params[:nud], params[:birth_date])
    if @instance.errors.present?
      return render_json_error(:unprocessable_entity, :invalid_birth_date) if @instance.errors.messages[:birth_date]
      return render_json_error(:not_found, :invalid_nud) if @instance.errors.messages[:nud]
    end

    render(jsonapi: @instance, serializer: @serializer)
  end

  def switch
    @instance = Hermes::Applicant.find(params[:id]) if params[:id].present?
    return render_json_error(:not_found, :instance_not_found) unless @instance

    @target = User.find_by(main_applicant_id: @instance.id)
    @target = User.find_by(email: @instance.email) unless @target
    return render(json: { success: false, message: 'Utilisateur target non trouvé.' }, status: :not_found) unless @target
    return render nothing: true, status: :not_found unless @user

    if current_user_roles.include?('entreprise')
      delegated_switch_response
    else
      render json: {
        success: false,
        message: 'Votre role doit être \'entreprise\' pour effectuer cette action.'
      }, status: :unauthorized
    end
  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
  end

  private

  def user_can_destroy?
    (@instance.user_id.to_s == current_user.id.to_s) ||
      current_user.has_role?('Administrator') ||
      current_user.has_role?('Manager') ||
      current_user.has_role?('Superadmin')
  end

  def delegated_switch_response
    params.require(:entity_id)
    entity = Hermes::Entity.readable_by(@user).find_by(id: params[:entity_id])
    return render(json: { success: false, message: 'Entity not found' }, status: :not_found) unless entity
    return render(json: { success: false, message: 'No active housing_request for this applicant' }, status: :not_found) unless @instance.first_active_request

    search_params = {
      entity_id: entity.id, 
      housing_request_id: @instance.first_active_request&.id
    }
   
    entity_form = Hermes::HousingRequestEntityForm.find_by(search_params)
    return render(json: { success: false, message: 'delegation not found' }, status: :not_found) unless entity_form
    if entity_form.present? && entity_form.delegation && entity_form.delegation_signed
      success_switch_response
    else
      render json: {
        success: false,
        message: 'delegation not available or not signed'
      }, status: :unauthorized
    end
  end

  def success_switch_response
    token = @target.generate_jwt(true_user_id: @user.id.to_s)
    render json: {
      success: true,
      info: t(:registered),
      email: @target.email,
      login: @target.login,
      jwt_token: token
    }, status: :ok
  end
end
