module Hermes
  module Api
    module Dmo
      class PicturesController < Hermes::Api::Dmo::ApplicationController
        before_action :check_upload_file, only: [:create]
        
        def create
          auth_roles_can_upload = %w[
            rdsl
            lessor
            collaborateur offer logement
          ]
          return render_json_error :unauthorized, :cannot_upload unless is_admin_or_super_admin || auth_roles_can_upload.any? {|role| current_user_roles.include?(role)}

          @instance = Hermes::Picture.new(user_id:  @user.id, writers: [@user.id])
          blob = @file.read
          @instance.set_blob(blob)

          return respond_with_errors(@instance) unless @instance.save
          render(jsonapi: @instance, serializer: Hermes::PictureSerializer, status: :created)
        end
      
        private 

        def set_instance
          @instance = Hermes::Picture.find_by(id: params[:id])
        end

        def user_can_destroy?
          is_admin_or_super_admin || @instance&.housing_offer&.readable_by?(@user) || @instance&.writable_by?(@user) 
        end 
        
        def check_upload_file
          return render_json_error :unprocessable_entity, :no_file_in_the_request unless @file = params[:file]
          
          begin 
            Magick::Image.ping(@file.path).first.nil?
          rescue Magick::ImageMagickError
            return render_json_error :unsupported_media_type, :not_an_image 
          end
        end 
      end
    end
  end
end