# frozen_string_literal: true

module Hermes
  # Entity serializer
  class EntitySerializer < Hermes::ApplicationSerializer
    attributes %i[
      als_id
      title
      siret
      siren
      is_root
      root_id
      parent_id
      id_rex
      id_cristal
      auto_validation
      auto_request_validation
      priorities_enabled
      in_peec
      achieved_goal
      spent_on_completed_requests
      spent_on_ongoing_requests
      spent_on_other
      annual_stock
      total_spent
      available_points
      lessor_id
      group_id
      supervisor_group_id
      manager_group_id
      current_user_right
      manual_mode
      is_master
      cre_email
      cre_first_name
      cre_last_name
    ]

    type 'entities'

    def added_to_object_cache_key
      current_user&.id
    end

    def manual_mode
      object&.manual_mode if object.respond_to? :manual_mode
    end

    def lessor_id
      object.lessor_id&.to_s
    end

    def group_id
      object.group_id&.to_s
    end

    def supervisor_group_id
      object.supervisor_group_id
    end

    def manager_group_id
      object.manager_group_id
    end

    def current_user_right
      return unless current_user
      right = Hermes::UserRight.find_by(user_id: current_user.id, entity_id: id)
      return if !right
      {
        id: id,
        right: right.right,
        type: right.type,
        title: right.title,
        group_id: right.group_id,
        entity_id: right.entity_id,
        lessor_id: right.lessor_id,
        user_id: right.user_id,
      } 
    end
  end
end
