require 'rails_helper'
require 'vcr_config'

RSpec.describe Hermes::RequiredDocument, type: :model do
  let!(:user1) {create(:user)}
  let!(:user2) {create(:user)}
  let!(:entity1) {create(:entity, siret: '00000000000001' )}
  let!(:applicant1) {
    create(
      :applicant,
      user: user1,
      monthly_charges: {"PAV" => 200},
      nationality: "1",
      user_id: user1.id,
      first_name: "robin",
      last_name: "caramaschi",
      birth_date: Date.current,
      main_employer_siret: entity1.siret,
      amount_year_minus_two: 20000,
    )
  }
  
  
  it 'has a valid factory' do
    expect(FactoryBot.build(:housing_request_folder)).to be_valid
  end


  let!(:housing_request1) do
    create(
      :housing_request,
      is_active: true,
      main_applicant: applicant1,
      user_id: user1.id,
      entity_status: 'EN COURS DE SAISIE',
      form_status: 'complete',
      seeked_departments: ['21']
    )
  end
  let!(:nud1) do
    create(
      :nud,
      housing_request: housing_request1,
      department: '12',
      creation_date: '2016-23-14T14:53:22',
      unique_number: "A1253EF84F3"
    )
  end
  describe "les droits sont transmis" do
    it "is good " do
      hrf = housing_request1.reload.housing_request_folder
      required_document = housing_request1.reload.housing_request_folder.required_documents[0]
      expect(housing_request1.user_id).to eq(user1.id)
      expect(hrf.user_id).to eq(user1.id)
      expect(required_document.user_id).to eq(user1.id)
    end
  end

 

   

  describe 'declared' do 
    let!(:rd) { housing_request1.housing_request_folder.required_documents.first}

    it 'not set by default' do
      rd.reload
      expect(rd.declared).to eq(nil)
      expect(rd.status).to eq('Pièce manquante')
    end

    it 'when true' do
      expect{rd.update(declared: true)}.to change{rd.reload.status}.from('Pièce manquante').to('Pièce validée')
    end

    it 'when false' do
  rd.declared = true
  rd.save
      expect{rd.update(declared: false)}.to change{rd.reload.status}.from('Pièce validée').to('Pièce manquante')
    end

  end 
end 